// Multiplexing a Common Row Cathode LED matrix
// Details on:
// www.homofaciens.de/technics-physical-computing-LED-matrix_en.htm


// Define the pin numbers for the rows
#define ROW_1   7
#define ROW_2   6
#define ROW_3   5
#define ROW_4   4
#define ROW_5   3
#define ROW_6   2

// Define the pin numbers for the columns
#define COL_1   8
#define COL_2   9
#define COL_3  10
#define COL_4  11
#define COL_5  12
#define COL_6  13

// Define number of columns and rows
#define SIZE_COLS 6
#define SIZE_ROWS 6

// Define the arrays of the pin numbers of Rows and columns
int rows[SIZE_ROWS] = {ROW_1, ROW_2, ROW_3, ROW_4, ROW_5, ROW_6};
int cols[SIZE_COLS] = {COL_1, COL_2, COL_3, COL_4, COL_5, COL_6};

// The matrix sets the displayed pattern
// 1 = LED is ON, 0 = LED is OFF
// example pattern displays character "O"
int matrix[SIZE_ROWS][SIZE_COLS]{
  {0,0,1,1,0,0},
  {0,1,0,0,1,0},
  {1,0,0,0,0,1},
  {1,0,0,0,0,1},
  {0,1,0,0,1,0},
  {0,0,1,1,0,0},
};


int i, j;

// Setup routine runs at startup or when you press reset:
void setup() {
  // initialize the digital pin as an output.
  for(i=0; i < SIZE_ROWS; i++){
    pinMode(rows[i], OUTPUT);
    // To turn OFF all rows, set them to HIGH
    digitalWrite(rows[i], HIGH);
  }
  for(i=0; i < SIZE_COLS; i++){
    pinMode(cols[i], OUTPUT);
    // To turn OFF all columns, set them to LOW
    digitalWrite(cols[i], LOW);
  }

}

// the loop routine runs over and over again forever:
void loop() {

  // Loop through all rows
  for(i=0; i < SIZE_ROWS; i++){
    // Loop through all columns
    for(j=0; j < SIZE_COLS; j++){
      // Set the required LEDs in the current row to HIGH
      if(matrix[i][j] == 1){
        digitalWrite(cols[j], HIGH);
      }
      else{ // all other LEDs to LOW
        digitalWrite(cols[j], LOW);
      }
    }
    // Now activate the current line with LOE signal
    digitalWrite(rows[i], LOW);
    delay(1);
    // Deactivate current line after the delay with HIGH signal
    digitalWrite(rows[i], HIGH);
  }

}

