//Send text to Arduino Uno, project LED matrix
//
// compile with:
// gcc sendText.c -o sendText -lm
//
// For details see:
// https://homofaciens.de/technics-physical-computing-LED-matrix_en.htm

#include <stdio.h>
#include <termios.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>


#define BAUDRATE B115200
//#define ARDUINOPORT "/dev/ttyUSB0"
#define ARDUINOPORT "/dev/ttyACM0"


int fd = 0;
struct termios TermOpt;



//+++++++++++++++++++++++ Start readport +++++++++++++++++++++++++++++++
char  readport(void){
  int n;
  char buff;
  n = read(fd, &buff, 1);
  if(n > 0){
    return buff;
  }
  return 0;
}
//---------------------- End readport ----------------------------------

//+++++++++++++++++++++++ Start sendport +++++++++++++++++++++++++++++++
void sendport(unsigned char ValueToSend){
  int n;

  n = write(fd, &ValueToSend, 1);

  if (n < 0){
    printf("write() of value failed!\n");
  }

}
//---------------------- End sendport ----------------------------------

//+++++++++++++++++++++++ Start openport +++++++++++++++++++++++++++++++
void openport(void){

    fd = open(ARDUINOPORT, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1)  {
      printf("init_serialport: Unable to open port!\n");
    }

    if (tcgetattr(fd, &TermOpt) < 0) {
      printf("init_serialport: Couldn't get term attributes!\n");
    }
    speed_t brate = BAUDRATE; // let you override switch below if needed

    cfsetispeed(&TermOpt, brate);
    cfsetospeed(&TermOpt, brate);

    // 8N1
    TermOpt.c_cflag &= ~PARENB;
    TermOpt.c_cflag &= ~CSTOPB;
    TermOpt.c_cflag &= ~CSIZE;
    TermOpt.c_cflag |= CS8;
    // no flow control
    TermOpt.c_cflag &= ~CRTSCTS;

    TermOpt.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    TermOpt.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

    TermOpt.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    TermOpt.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    TermOpt.c_cc[VMIN]  = 0;
    TermOpt.c_cc[VTIME] = 20;

    if( tcsetattr(fd, TCSANOW, &TermOpt) < 0) {
      printf("init_serialport: Couldn't set term attributes!\n");
    }

}
//---------------------- End openport ----------------------------------




//######################################################################
//################## Main ##############################################
//######################################################################

int main(int argc, char **argv){

  openport();


  printf("Enter text: ");
  char *TextLine = NULL;
  size_t len = 0;
  ssize_t lineSize = 0;
  lineSize = getline(&TextLine, &len, stdin);
  TextLine[lineSize-1]='\0';
  printf("Your text is \"%s\", length = %zu chars.\n", TextLine, lineSize - 1);

  for(int i = 0; i < lineSize - 1; i++){
    printf("Sending %c ...", TextLine[i]);
    sendport(TextLine[i]);
    printf("...sent.\n");
  }
  sendport(13);
  printf("\n\n...done!\n");

  close(fd);


  return 0;
}
